//============================================================================
// Name        : factorize.cpp
// Author      : Henrik Nielsen
// Version     :
// Copyright   : Your copyright notice
// Description :
//============================================================================

#include <iostream>
#include <string>
#include <cstdlib> // for exit(0)
#include <vector>
#include <map>
#include <algorithm> //for min(x,y)

using namespace std;

typedef vector<string> vectorS;
typedef vector<int> vectorI;
typedef map<vectorI, int> KoeffMap; // map exponents on unknown coefficients to their number-coefficients
typedef map<vectorI, KoeffMap> ExpMap; // map exponents to their coefficients

//Take a comma-separated list and put elements in vector
void populate(vectorS &vec, string str, char separator = ','){
	int elementStart = 0;
	int elementEnd;
	while(     ( elementEnd = str.find(separator, elementStart) )    != string::npos ){
		vec.push_back( str.substr(elementStart, elementEnd - elementStart) );
		elementStart = elementEnd+1;
	}

	vec.push_back( str.substr(elementStart, str.length() - elementStart ) );

}

//Increase delim at least one until the delim's character in poly is '+' or '-' (or set it to string::npos)
int nextDelimiter(int delim, string poly){
	int temp = poly.find('+', delim + 1);
	delim = poly.find('-', delim + 1);

	if (temp == string::npos) return delim;
	if (delim == string::npos) return temp;

	if( delim < temp ) return delim;
					else return temp;
}

int comp(string pConstant, string constant){
	int hat = pConstant.find('^');
	if( hat == string::npos ){
		if( constant.compare(pConstant) != 0 ) return 0;
		return 1;
	}
	if( constant.compare( pConstant.substr(0, hat) ) != 0) return 0;
	string exp = pConstant.substr( hat+1, pConstant.length() - hat );
	return atoi( exp.c_str() );
}

int disectFactor(string factor, vectorS constant, vectorS variable, vectorI &koeffExp, vectorI &exponent){
	int sign = 1;
	if( factor.find("-") == 0 ){
		sign = -1;
		factor = factor.substr(1, factor.length() - 1 );
	}
	for( int j = 0; j < constant.size(); ++j ){
		int temp = comp(factor, constant[j]);
		if( temp != 0 ){ //comp returnerer koefficienten der skal lægges til
			koeffExp.at(j) += temp;
			return sign;
		}
	}
	for( int j = 0; j < exponent.size(); ++j ){
		int temp = comp(factor, variable.at(j));
		if( temp != 0 ){
			exponent.at(j) += temp;
			return sign;
		}
	}
	return sign*atoi( factor.c_str() );
}

// leadsign: true hvis der SKAL fortegn foran koefficienten
// returner: false hvis koefficienten blot var et fortegn
//				(i så fald skal kalder ikke bruge et *)
bool printKoeffSingle(vectorI expVec, int frontFactor, bool leadSign, vectorS constant){
	bool firstFactor = true;

	if( leadSign && frontFactor > 0 ) cout << '+';
	if( frontFactor != 1 ){
		if( frontFactor == -1 )	cout << '-';
		else if( frontFactor != 1 ){
			cout << frontFactor;
			firstFactor = false;
		}
	}
	for( int j = 0; j < expVec.size(); ++j ){ //Print koefficient-eksponentvektoren
		if( expVec.at(j) == 0 ) continue;
		if( !firstFactor ) cout << "*";
		firstFactor = false;
		cout << constant.at(j);
		if( expVec.at(j) > 1 ) cout << "^" << expVec.at(j);
	}

	return !firstFactor;
}


// leadsign: true hvis der SKAL fortegn foran koefficienten
// returner: false hvis koefficienten blot var et fortegn
//				(så hvis der returneres true skal de næste faktorer *'es på)
bool printKoeff(KoeffMap koeffMap, vectorS constant, bool leadSign){

	KoeffMap::iterator it = koeffMap.begin();
	if( koeffMap.size() == 1 ){
		return printKoeffSingle(it->first, it->second, leadSign, constant);
	}

	//Assume koeffMap.size() > 1
	if( leadSign ) cout << '+';
	cout << '(';

	printKoeffSingle(it->first, it->second, false, constant); //no lead sign in parenthesis
	while( ++it != koeffMap.end() ){
		printKoeffSingle(it->first, it->second, true, constant);
	}
	cout << ')';

	return true;
}


bool printVars(vectorI expVec, bool printMult, vectorS variable) {
	for( int i = 0; i < expVec.size(); ++i ){ //Print koefficient-eksponentvektoren
		if (expVec.at(i) == 0) continue;
		if( printMult ) cout << "*";
		printMult = true;
		cout << variable.at(i);
		if( expVec.at(i) > 1 ) cout << "^" << expVec.at(i);
	}
	return printMult;
}


void printMap(ExpMap p, vectorS constant, vectorS variable){
	//Print the factorized poly
	if( p.empty() ){
		cout << "0, " << endl;
		return;
	}
	vectorI expVec;
	bool printMult;


	// First term --------------------
	ExpMap::iterator expIt = p.begin();
	printMult = printKoeff(expIt->second, constant, false); //No leading sign for first koeff.
	printMult = printVars(expIt->first, printMult, variable);
	if( !printMult ) cout << '1';

	// n'th terms (n > 1) -----------------
	while( ++expIt != p.end() ){
		printMult = printKoeff(expIt->second, constant, true); //Leading sign for following koeff.
		printMult = printVars(expIt->first, printMult, variable);
		if( !printMult ) cout << '1';
	}
	cout << endl;
}


void createExpMap(ExpMap &expMap, string poly, vectorS constant, vectorS variable) {
	//Split in terms
	ExpMap::iterator it;
	int elementStart = 0;
	int elementEnd = nextDelimiter(elementStart, poly);
	vectorS term; //This will hold the terms as strings

	while( elementEnd != string::npos ){
		term.push_back(poly.substr(elementStart, elementEnd-elementStart));
		if( poly.at(elementEnd) == '+' ) elementStart = elementEnd+1;
									else elementStart = elementEnd;
		elementEnd = nextDelimiter(elementEnd+1, poly);
	}
	term.push_back( poly.substr(elementStart, poly.length() - elementStart ) );

	for( int i = 0; i < term.size(); ++i ){
		vectorI exponent;
		vectorI koeffExp;
		exponent.resize(variable.size(), 0);
		koeffExp.resize(constant.size(), 0);

		vectorS factor;
		populate(factor, term[i], '*');
		int frontFactor = 1;
		for( int j = 0; j < factor.size(); ++j ){
			frontFactor *= disectFactor(factor.at(j), constant, variable, koeffExp, exponent);
		}



		int temp = expMap.count(exponent);
		if( temp > 0 ){ //exponent found in expMap.
			KoeffMap &koeffMap = expMap[exponent];
			KoeffMap::iterator it = koeffMap.begin();
			if( koeffMap.count(koeffExp) > 0 ){
				koeffMap[koeffExp] += frontFactor;
			} else {
				koeffMap.insert ( std::pair<vectorI,int>(koeffExp, frontFactor) );
			}
			continue;
		}

		KoeffMap koeffMap;
		koeffMap.insert( std::pair<vectorI,int>(koeffExp, frontFactor) );
		expMap.insert( std::pair<vectorI,KoeffMap>(exponent, koeffMap) );
		it = expMap.begin();
	}


	return;
}


int main(int argc, char *argv[]) {
	if( argc != 2 ){
		cout << "Need argument: number of constants." << endl;
		return -1;
	}
	int noConsts = atoi(argv[1]);

	//get data from the standard input
	string str, line;
    while(std::getline(std::cin, line)) str.append(line);

	if(str.find("Q[") != 0){ // If "Q<" not in beginning...
		cout << "Bad input: Must start with 'Q['" << endl;
		exit(0);
	}

	size_t varEnd = str.find("]{");
	if(varEnd == string::npos){ //If ">[" not found...
		cout << "Bad input: Couldn't find ']{'" << endl;
		exit(0);
	}


	size_t polyEnd = str.find("}");
	if(polyEnd == string::npos){ //If "}" not found...
		cout << "Bad input: Couldn't find '}'" << endl;
		exit(0);
	}

	string subStr;
	vectorS constant, variable, poly;

	subStr = str.substr(2, varEnd-2); // .substr(beginning, length)
	int constEnd = 0;
	while( noConsts-- > 0 ){
		int temp = subStr.find(',', constEnd+1);
		if( temp == string::npos ) constEnd = subStr.length();
		else constEnd = temp;

	}

	populate(constant, subStr.substr(0, constEnd));
	populate(variable, subStr.substr(constEnd+1, varEnd-constEnd-3));

	subStr = str.substr(varEnd+2, polyEnd-varEnd-2);
	populate(poly, subStr);



	for( int i = 0; i < poly.size(); ++i ){
		ExpMap p;
		createExpMap(p, poly[i], constant, variable);

		printMap(p, constant, variable);

	}

	return 0;
}
