//============================================================================
// Name        : initialGen.cpp
// Author      : Henrik Nielsen
// Version     :
// Copyright   : 
// Description :
//============================================================================

#include <iostream>
#include <cstdlib>
#include <string>
#include <vector>
#include <map>
using namespace std;

typedef vector<string> vectorS;
typedef vector<int> vectorI;
typedef map<vectorI, int> PolyRep;


//Take a comma-separated list and put elements in vector
void populate(vectorS &vec, string str, char separator = ','){
	int elementStart = 0;
	int elementEnd;
	while(     ( elementEnd = str.find(separator, elementStart) )    != string::npos ){
		vec.push_back( str.substr(elementStart, elementEnd - elementStart) );
		elementStart = elementEnd+1;
	}

	vec.push_back( str.substr(elementStart, str.length() - elementStart ) );
}

//Increase delim at least one until the delim's character in poly is '+' or '-' (or set it to string::npos)
int nextDelimiter(int delim, string poly){
	int temp = poly.find('+', delim + 1);
	delim = poly.find('-', delim + 1);

	if (temp == string::npos) return delim;
	if (delim == string::npos) return temp;

	if( delim < temp ) return delim;
					else return temp;
}


void addTerm(PolyRep &pRep, string term, vectorS var){
	vectorS factor;
	populate(factor, term, '*');

	int intFactor;
	string &frontFactor = factor[0];

	if( frontFactor[0] == '-' ){ //Hvis - står foran første faktor, omend det er et tal eller en variabel.
		frontFactor.erase(0,1);
		intFactor = -1;
	} else {
		intFactor = 1;
	}

	int i = 0;
	while( frontFactor[i] >= '0' && frontFactor[i] <= '9' ) ++i;
	if( frontFactor[i] == '\0' ){ //Hvis frontFactor er et tal, og ikke en variabel
		intFactor *= atoi( frontFactor.c_str() );
		factor.erase( factor.begin() );
	}

	//int-faktoren er nu fundet og fjernet fra factor. Nu skal eksponentvektoren findes
	vectorI expVec( var.size(), 0 ); //Start with (var.size()) 0's
	for( int i = 0; i < factor.size(); ++i ){

		string v = factor[i];
		int hat, exp;
		hat = v.find('^');

		if( hat != string::npos ){
			exp = atoi( v.c_str()+hat+1 );
			v.erase(hat);
		} else {
			exp = 1;
		}

		// v should now be equal one element from var
		int j = 0;
		while( v.compare(var[j]) != 0 ) ++j;
		expVec[j] += exp;
	}

	if( pRep.count(expVec) > 0 ){
		pRep[expVec] += intFactor;
	} else {
		pRep.insert( std::pair<vectorI,int>(expVec, intFactor) );
	}

	return;
}


void findPolyRep(PolyRep &p, string poly, vectorS variable ){
	vectorS term; //This will hold the terms as strings
	{
		int elementStart = 0;
		int elementEnd = nextDelimiter(elementStart, poly);

		while( elementEnd != string::npos ){
			term.push_back(poly.substr(elementStart, elementEnd-elementStart));
			if( poly.at(elementEnd) == '+' ) elementStart = elementEnd+1;
									else elementStart = elementEnd;
			elementEnd = nextDelimiter(elementEnd+1, poly);
		}
		term.push_back( poly.substr(elementStart, poly.length() - elementStart ) );
	}

	//Tilføj hver term til p.
	for( int i = 0; i < term.size(); ++i ) addTerm(p, term[i], variable);

	return;
}

int dot(vectorI a, vectorI b){
	int res = 0;
	for( int i = 0; i < a.size(); ++i ) res += a[i]*b[i];
	return res;
}


void transformToInitial(PolyRep &p, vectorI exp){
	//First, find the degree
	int deg = 0;
	for( PolyRep::iterator it = p.begin(); it != p.end(); ++it ){
		int temp = dot(it->first, exp);
		if( deg < temp ) deg = temp;
	}

	//Then, remove terms with lover degree.
	PolyRep::iterator it = p.begin();
	while( it != p.end() ){
		PolyRep::iterator itNext = it; ++itNext;
		if( dot(it->first, exp) < deg) p.erase(it); //Herved bliver it ugyldig (derfor blev dens næste gemt i itNext).
		it = itNext;
	}
}

PolyRep reducePoly(PolyRep p){
	//Find largest common factor
	PolyRep::iterator it = p.begin();
	vectorI commonFactor = it->first;
	while( ++it != p.end() ){
		vectorI termExp = it->first;
		for( int i = 0; i < commonFactor.size(); ++i ){
			if( commonFactor[i] > termExp[i] ) commonFactor[i] = termExp[i];
		}
	}


	PolyRep res;

	it = p.begin();
	int temp = 0;
	while( it != p.end() ){
		++temp;
		if( temp == 3 ){

		}
		vectorI termExp = it->first;
		for( int i = 0; i < commonFactor.size(); ++i ) termExp[i] -= commonFactor[i];
		res.insert( std::pair<vectorI,int>(termExp, it->second) );
		++it;
	}

	return res;
}

// leadsign: true hvis der SKAL fortegn foran koefficienten
// returner: false hvis koefficienten blot var et fortegn
//				(så hvis der returneres true skal de næste faktorer *'es på)
bool printKoeff(int koeff, bool leadSign){
	if( leadSign && koeff > 0 ) cout << '+';

	if( koeff == 1 ) return false;
	if( koeff == -1 ){
		cout << '-';
		return false;
	}

	cout << koeff;
	return true;
}

//returner: false hvis 'næste' faktor ikke behøver et '*' foran
bool printVars(vectorI expVec, bool printMult, vectorS variable) {
	for( int i = 0; i < expVec.size(); ++i ){ //Print koefficient-eksponentvektoren
		if (expVec.at(i) == 0) continue;
		if( printMult ) cout << "*";
		printMult = true;
		cout << variable.at(i);
		if( expVec.at(i) > 1 ) cout << "^" << expVec.at(i);
	}
	return printMult;
}


void printPoly(PolyRep p, vectorS variable){
	//Print the factorized poly
	if( p.empty() ){
		cout << "0, " << endl;
		return;
	}
	vectorI expVec;
	bool printMult;


	// First term --------------------
	PolyRep::iterator expIt = p.begin();
	printMult = printKoeff(expIt->second, false); //No leading sign for first koeff.
	printMult = printVars(expIt->first, printMult, variable);
	if( !printMult ){
		//Næste faktor (som der ikke er nogen af) behøver ikke et foranstillet '*'
		//Dette betyder at ingen faktor overhovedet er printet (kun evt et fortegn.)
		cout << "1";
	}

	// n'th terms (n > 1) -----------------
	while( ++expIt != p.end() ){
		printMult = printKoeff(expIt->second, true); //Leading sign for following koeff.
		printMult = printVars(expIt->first, printMult, variable);
		if( !printMult ) cout << "1";
	}
}


int main(int argc, char *argv[]) {
	if( argc < 2 ){
		cout << "Need argument: exponentvector." << endl;
		return -1;
	}

	//////////////////////////////////////////////////////////////////////////////////////
	// Put argument on the form '[1,2,3]' into vector exp //
	string str = argv[1]; //exponentvectoren givet m. argument
	str = str.substr(1, str.length()-2); //Udtag alt undtagen '[' og ']'

	vectorI exp;
	{// Scope for expStr
		vectorS expStr;
		populate(expStr, str);
		for( int i = 0; i < expStr.size(); ++i ) exp.push_back(atoi( expStr[i].c_str() ));
	}

	// Read other arguments
	bool reduce = false;
	bool ring = false;
	if( argc >= 3 ){
		str = argv[2];
		reduce = (str.compare("--reduce") == 0);
		ring = (str.compare("--ring") == 0);
	}
	if( argc >= 4 ){
		str = argv[3];
		reduce = reduce || (str.compare("--reduce") == 0);
		ring = ring || (str.compare("--ring") == 0);
	}

	// Store data from standard input appropriately //
	str.clear();
	vectorS poly, variable;
	string line;
    while(std::getline(std::cin, line)) str.append(line);

	if(str.find("Q[") != 0){ // If "Q<" not in beginning...
		cout << "Bad input: Must start with 'Q['" << endl;
		exit(0);
	}
	size_t varEnd = str.find("]{");
	if(varEnd == string::npos){ //If ">[" not found...
		cout << "Bad input: Couldn't find ']{'" << endl;
		exit(0);
	}
	size_t polyEnd = str.find("}");
	if(polyEnd == string::npos){ //If "}" not found...
		cout << "Bad input: Couldn't find '}'" << endl;
		exit(0);
	}

	if( ring ){
		cout << "Q[" << str.substr(2, varEnd-2) << "]{" << endl;
	}


	populate(variable, str.substr(2, varEnd-2));
	populate(poly, str.substr(varEnd+2, polyEnd-varEnd-2));

	if(variable.size() != exp.size()){
		cout << "Dimension of standard input (" << variable.size() << ") and vector-argument (" << exp.size() << ") doesn't match" << endl;
		exit(0);
	}

	int i = 0;
	while( true ){
		PolyRep p;
		findPolyRep(p, poly[i], variable);
		transformToInitial(p, exp); // Transform p to its initial polynomial w.r.t. exp
		if( reduce ) p = reducePoly(p);
		printPoly(p, variable);

		++i;
		if(i == poly.size()) break;

		cout << ',' << endl;
	}

	if( ring ) cout << '}';
	cout << endl;

	return 0;
}
